# Copyright (c) 2007, Northwestern Medical Faculty Foundation, Inc.
# 
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
# 
#   * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. 
#   * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution. 
#   * Neither the name of Northwestern Medical Faculty Foundation, Inc. nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission. 
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

#This controller governs all user-level actions related to their own user accounts.
#It contains actions for login/logout, user creation, and password modification.

class AuthorizeController < ApplicationController
  require 'digest/sha1'   	
  model :user
  
  #Attempt to log the user in from the form data.
  #If the uname/pw combination is not found, it flashes such and allows
  #the user to try again.
  def login
    if @request.get?
      session[:user_id] = nil
      session[:level] = nil
      @user = User.new
    else
      @user = User.new(params[:user])
      logged_in_user = @user.try_to_login
      if logged_in_user
	  session[:user_id] = logged_in_user.id
	  session[:level] = logged_in_user.user_level
	  redirect_to(:controller => 'metarep', :action => 'browser')  
      else
	flash[:notice] = "Invalid user/password combintation"
      end
    end
  end

  #Allows the logged in user to logout. The session data regarding user info is cleared.
  def logout
    flash[:notice] = "You have successfully been logged out."
    session[:user_id] = nil
    session[:level] = nil
    redirect_to(:controller => 'metarep', :action => 'browser')
  end
  
  #This is the preferences menu. It currently only links the user to the change password action.
  #If the user is not logged in, it redirects the user to login.
  def index
    unless session[:user_id]
      flash[:notice] = "Please log in"
      redirect_to(:action => 'login')
    end 
  end
  
  #Create memory space for a new user when the new page is visited
  def new
    @user = User.new
  end
  
  #Store the new user in the users table. If it fails for whatever reason, nothing
  #will happen and the user will be able to try again.
  def create
      @user = User.new(:email => params[:user][:email],
				:password => params[:user][:password],
				:password_confirmation => params[:user][:password_confirmation],
				:user_level => 1)
      @user.first_name = params[:user][:first_name]
      @user.last_name = params[:user][:last_name]
      if @user.save
        session[:user_id] = @user.id
        session[:level] = @user.user_level
        flash[:notice] = "Account creation successful."
        redirect_to :controller => 'metarep', :action => 'browser'
      else
        render :action => 'new'
      end
  end
  
  #Allows user to enter his or her e-mail address. The action will find the user with that e-mail.
  #If the user is not found, flash indicates such.
  #If the user is found, resets password to a random 8-digit number, then e-mails
  #the user telling him or her the password. Also, sets the user's flag to force
  #a password change upon next login.
  def reset_password
    flash[:notice] = nil
    if request.post?
    @user = User.find_by_email(params[:user][:email])
      if @user
        @new_password = newpass(8)
        @user.password = @new_password
	@user.flag = 'RESET'
        @user.save
        Notifier.deliver_reset_password_email(@user, @new_password)
	flash[:notice] = "Password reset and e-mail with new password sent."
        redirect_to :controller => 'metarep', :action => 'browser'
      else
        flash[:notice] = "User with this e-mail address not found."
      end
    end
  end
  
  #Allows a logged-in user to change his or her password by entering it and confirming it.
  #If the user is not logged in, it will redirect the user to login.
  #If the passwords don't match, the action flashes such.
  def change_password
    if session[:user_id]
      if request.post?
	@user = User.find(session[:user_id])      
	if params[:user][:password] == params[:user][:password_confirmation]
	  @user.password = params[:user][:password]
	  @user.flag = nil
	  @user.save
	  flash[:notice] = "Password change successful."
	  redirect_to :controller => 'metarep', :action => 'browser'
        else
	  flash[:notice] = "Password does not match the confirmation."	
        end
      end
    else
      flash[:notice] = "Please log in."
      redirect_to :action => 'login'
    end
  end
  
  #Used by reset_password to generate a random password for the user.
  def newpass( len )
    chars = ("0".."9").to_a
    newpass = ""
    1.upto(len) { |i| newpass << chars[rand(chars.size-1)] }
    return newpass
  end
end